//
//  FGHudCircleCell.m
//  FGBseHUD
//
//  Created by Franco on 15/02/06.
//
//  Copyright 2006 Softech di Germinara Francesco
//
//  www.germinara.it
//  info@germinara.it
//
//  All rights reserved.
/////////////////////////////////////////////////////////

#import "FGHudCircleCell.h"
#import "FGNSBezierPathAddition.h"

 static float getValueIn360(float ang){
    while (ang < 0.0) ang += 360.0;
    while (ang >= 360.0) ang -= 360.0;
    return ang;
}

 static float getAngleFromPoint(float x, float y){
    double result = atan2(y, x) * (180 / M_PI);
    return getValueIn360(result); // restituisce il valore tra  in 0-360 
}

@implementation FGHudCircleCell

- (id)init{
    self = [super init];
    if (self) {
        aFillColor=[[NSColor yellowColor] retain];  //Colore della selezione
        aBorderColor=[[NSColor blackColor] retain];  //Colore della selezione
    	[self setDoubleValue:0.0];
        [self setShowsFirstResponder:YES];
        [self setContinuous:YES];
        bUseGradient=NO;
        bUseCustomKnob=NO;
	    aImageButton=[[NSImage  imageNamed:@"fgcircleknob.tiff"] retain];
        aImageButtonPressedOn=[[NSImage  imageNamed:@"fgcircleknobpressed.tiff"] retain];
		
    }
    return self;
}

-(void) dealloc {
	[aFillColor release];
	[aBorderColor release];
	[aImageButton release];
	[aImageButtonPressedOn release];
	[super dealloc];
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{

 NSView *cv = [self controlView];
[cv setToolTip:[self stringValue]];

 NSRect rcInvalidate= [[self controlView] bounds];
 aHudView=(FGHUDView *)[[super controlView] superview];

  foreColor=[aHudView clrTitleBar];
  backColor=[aHudView clrBodyBar];

  [backColor set];
  NSRectFill(rcInvalidate);

    NSBezierPath *path = [NSBezierPath bezierPath];
    NSRect drawFrame = cellFrame;
    double theAngle = [self doubleValue];

    theAngle = getValueIn360(theAngle);
    if (cellFrame.size.width > cellFrame.size.height) {
        drawFrame.size.width = cellFrame.size.height;
    }
    if (cellFrame.size.height > cellFrame.size.width) {
        drawFrame.size.height = cellFrame.size.width;
    }
    drawFrame = NSInsetRect(drawFrame, 1.0, 1.0);
    center = NSMakePoint(cellFrame.size.width / 2.0 + cellFrame.origin.x,
                         cellFrame.size.height / 2.0 + cellFrame.origin.y);
    radius = center.x - 1.0;

 if(bUseCustomKnob==NO){

    [aFillColor set];

    [path setLineWidth:1.0];
    [path moveToPoint:center];
    [path appendBezierPathWithArcWithCenter:center radius:radius startAngle:0.0 endAngle:theAngle clockwise:NO];
    [path closePath];

    if(bUseGradient==YES)
     [path linearGradientFillWithStartColor:backColor endColor:foreColor];
    else
     [path fill];
    [aBorderColor set];
    [path setLineWidth:1.0];
    [path moveToPoint:center];
    [path appendBezierPathWithArcWithCenter:center radius:radius startAngle:0.0 endAngle:theAngle clockwise:NO];
    [path closePath];
    [path stroke];
    [path removeAllPoints];
    [path appendBezierPathWithOvalInRect:drawFrame];
    [path stroke];
  }else{
  int bPressed=[self isHighlighted];
  if(!bPressed){ 
   [self setImage:aImageButton];
  }
  if(bPressed ){
   [self setImage:aImageButtonPressedOn];
  }
 }

}


- (BOOL)startTrackingAt:(NSPoint)startPoint inView:(NSView *)controlView
{
    BOOL ret = [super startTrackingAt:startPoint inView:controlView];
    float theAngle = getAngleFromPoint(startPoint.x - center.x, startPoint.y - center.y);
    NSView *cv = [self controlView];
    [[cv window] makeFirstResponder:cv];
    [self setDoubleValue:theAngle];

//    NSLog(@"startTrackingAt - >Angolo: %2.f",theAngle);
    return ret;
}

- (BOOL)continueTracking:(NSPoint)lastPoint at:(NSPoint)currentPoint inView:(NSView *)controlView
{
    BOOL ret = [super continueTracking:lastPoint at:currentPoint inView:controlView];
    float theAngle = getAngleFromPoint(currentPoint.x - center.x, currentPoint.y - center.y);
    [self setDoubleValue:theAngle];

//    NSLog(@"continueTracking - >Angolo: %2.f",theAngle);

    return ret;
}

- (void)stopTracking:(NSPoint)lastPoint at:(NSPoint)stopPoint inView:(NSView *)controlView mouseIsUp:(BOOL)flag
{
    float theAngle = getAngleFromPoint(stopPoint.x - center.x, stopPoint.y - center.y);
    [self setDoubleValue:theAngle];
    [super stopTracking:lastPoint at:stopPoint inView:controlView mouseIsUp:flag];

 //   NSLog(@"stopTracking - >Angolo: %2.f",theAngle);

}

- (NSString *)stringValue
{
    return [NSString stringWithFormat:@"%.2f", value];
}

- (void)setStringValue:(NSString *)aString
{
    value = [aString doubleValue];
    [[self controlView] setNeedsDisplay:YES];
}

- (int)intValue
{
    return value;
}

- (void)setIntValue:(int)anInt
{
    value = anInt;
    [[self controlView] setNeedsDisplay:YES];
}

- (float)floatValue
{
    return value;
}

- (void)setFloatValue:(float)aFloat
{
    value = aFloat;
    [[self controlView] setNeedsDisplay:YES];
}

- (double)doubleValue
{
    return value;
}

- (void)setDoubleValue:(double)aDouble
{
    value = aDouble;
    [[self controlView] setNeedsDisplay:YES];
}

- (NSComparisonResult)compare:(id)otherCell
{
    double otherValue = [otherCell doubleValue];
    if (otherValue > value) return NSOrderedAscending;
    if (otherValue < value) return NSOrderedDescending;
    return NSOrderedSame;
}

-(void) setFillColor:(NSColor*) theFillColor{
 [aFillColor release];
 aFillColor=[theFillColor retain];
 [[self controlView] setNeedsDisplay:YES];
}

-(void) setBorderColor:(NSColor*) theBorderColor{
 [aBorderColor release];
 aBorderColor=[theBorderColor retain];
 [[self controlView] setNeedsDisplay:YES];
}

-(void) setUseGradient:(BOOL) bYesNo{
 bUseGradient=bYesNo;
 [[self controlView] setNeedsDisplay:YES];
}


- (void)setImage:(NSImage *)image
{

    // Save the state
    [NSGraphicsContext saveGraphicsState];

	NSImage *targetImage = [[NSImage alloc] initWithSize: [image size]];
	NSAffineTransform *trans = [NSAffineTransform transform];
    double theAngle = [self doubleValue];
    theAngle = getValueIn360(theAngle);	
    [image setFlipped: NO];	
  	[targetImage lockFocus];

	[trans translateXBy:[image size].width/2.0 yBy:[image size].height/2.0];
	[trans rotateByDegrees: theAngle];
	[trans translateXBy:-[image size].width/2.0 yBy:-[image size].height/2.0];
	[trans concat];

    [image drawAtPoint: NSZeroPoint fromRect: NSMakeRect(0,0,[image size].width,[image size].height) operation: NSCompositeSourceOver fraction: 1.0];

	[targetImage unlockFocus];
	[targetImage setScalesWhenResized: NO];

    [targetImage drawAtPoint: NSZeroPoint fromRect: NSMakeRect(0,0,[image size].width,[image size].height) operation: NSCompositeSourceOver fraction: 1.0];
 	[targetImage release];

    // Restore the state
    [NSGraphicsContext restoreGraphicsState];

}

- (void)seImageForNormalState:(NSImage *)imageNormal imageForPressedState:(NSImage*) imagePressed{
   [aImageButton release];
   [aImageButtonPressedOn release];
   aImageButton=[imageNormal retain];
   aImageButtonPressedOn=[imagePressed retain];
}

- (void) setUseCustomKnob:(BOOL) bYesNo{
 bUseCustomKnob=bYesNo;
 [[self controlView] setNeedsDisplay:YES];
}


@end
