//
//  FGHudLevelIndicatorCell.m
//  FGBseHUD
//
//  Created by Francesco on 2/17/06.
//
//  Copyright 2006 Softech di Germinara Francesco
//
//  www.germinara.it
//  info@germinara.it
//
//  All rights reserved.
/////////////

#import "FGHudLevelIndicatorCell.h"
#import "FGNSBezierPathAddition.h"


@implementation FGHudLevelIndicatorCell



- (id) init {
	self = [super initWithLevelIndicatorStyle:NSDiscreteCapacityLevelIndicatorStyle];
	if (self != nil) {
	    nType=FGContinuousCapacityLevelIndicatorStyle;
		double nAlpha=0.95;
		aBorderColor=[[NSColor colorWithCalibratedRed:0.10 green:0.10 blue:0.10 alpha:nAlpha]retain];
        aFillColor=[[NSColor colorWithCalibratedRed:0.21 green:0.21 blue:0.21 alpha:nAlpha] retain];
        aFillColorWarning=[[NSColor yellowColor] retain];  //Colore della selezione
        aFillColorCritical=[[NSColor redColor] retain];  //Colore della selezione
		theTitle=[[NSString alloc] initWithString:@""];;
		[self setDoubleValue:0];
		bUseGradient=YES;
		aImageButton=[[NSImage  imageNamed:@"FGBarraLed.tiff"] retain];
        nIncrement=1.0;
        nInterval=0.1;
		bEnableReverse=FALSE;
		nDirection=0;
	}
	return self;
}

-(void) dealloc {
	[aFillColor release];
	[aBorderColor release];
	[aFillColorWarning release];
	[aFillColorCritical release];
	[aImageButton release];
	[super dealloc];
}


//imposto il tipo di indicatore
-(void) setFGLevelIndicatorStyle:(FGLevelIndicatorStyle)levelIndicatorStyle {
 nType=levelIndicatorStyle;
 if((nType == FGContinuousCapacityLevelIndicatorStyleCustomColor)|| (nType == FGContinuousLedsLevelIndicatorStyle)){
  [self setLevelIndicatorStyle:FGRelevancyLevelIndicatorStyle];
 }else 
  [self setLevelIndicatorStyle:nType];
 [[self controlView] setNeedsDisplay:YES];
} 


- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
 
 if((nType == FGContinuousCapacityLevelIndicatorStyleCustomColor) || (nType == FGContinuousLedsLevelIndicatorStyle)){
   aHudView=(FGHUDView *)[[super controlView] superview];
   backColor=[aHudView clrBodyBar];
   foreColor=[aHudView clrTitleBar];
   [backColor set];
   NSRectFill(cellFrame);
   
   [aBorderColor set];
   NSFrameRect(cellFrame);
 
 
  NSRect rcFill=cellFrame;
  double nMinval=[self minValue];
  double nMaxval=[self maxValue];
  double nCurval=[self doubleValue];
  double nWarnval=[self warningValue];
  double nCritval=[self criticalValue];
  
  double scalaX=nMaxval - nMinval;
  scalaX=cellFrame.size.width / scalaX;
  
  int xAmp=nCurval * scalaX;
  rcFill.size.width=xAmp;
  
  if(nType == FGContinuousLedsLevelIndicatorStyle){
   [aImageButton setFlipped: YES];
   [aImageButton drawInRect:cellFrame fromRect:NSMakeRect(0,0,140,20) operation:NSCompositeDestinationOver fraction:1.0];		  
   [aImageButton drawInRect:rcFill fromRect:NSMakeRect(0,0,xAmp,20) operation:NSCompositeSourceOver fraction:1.0];		  
   [aImageButton setFlipped: NO];
  }else{
  if(bUseGradient){
   NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:rcFill cornerRadius:4];
   [path setFlatness:1.0];
   [path setLineWidth:1.0];
   [path bilinearGradientFillWithOuterColor:aFillColor innerColor:aBorderColor];
   [path stroke];
   
   if(nWarnval > 0.0 && nCurval > nWarnval ){
    path = [NSBezierPath bezierPathWithRoundedRect:rcFill cornerRadius:4];
    [path setFlatness:1.0];
    [path setLineWidth:1.0];
    [path bilinearGradientFillWithOuterColor:aFillColorWarning innerColor:aBorderColor];
    [path stroke];
   }

   if(nCritval > 0.0 && nCurval > nCritval && nCritval > nWarnval){
    path = [NSBezierPath bezierPathWithRoundedRect:rcFill cornerRadius:4];
    [path setFlatness:1.0];
    [path setLineWidth:1.0];
    [path bilinearGradientFillWithOuterColor:aFillColorCritical innerColor:aBorderColor];
    [path stroke];
   }
   
  }else{
   rcFill=NSInsetRect(rcFill,1,1);
   NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:rcFill cornerRadius:0];
   [path setFlatness:1.0];
   [path setLineWidth:1.0];
   [aFillColor set];
   [path fill];

   if(nWarnval > 0.0 && nCurval > nWarnval ){
   NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:rcFill cornerRadius:0];
   [path setFlatness:1.0];
   [path setLineWidth:1.0];
   [aFillColorWarning set];
   [path fill];
   }

   if(nCritval > 0.0 && nCurval > nCritval && nCritval > nWarnval){
    NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:rcFill cornerRadius:0];
    [path setFlatness:1.0];
    [path setLineWidth:1.0];
    [aFillColorCritical set];
    [path fill];
   }
 }
 
 if([theTitle length]>0){
  [self drawText:cellFrame];
 }
}
 
 return;
 }

 [super drawWithFrame:cellFrame inView:controlView];
}

- (void) setUseGradient:(BOOL) bYesNo{
  bUseGradient=bYesNo;
  [[self controlView] setNeedsDisplay:YES];
}

- (void) setFillColor:(NSColor*) aColor{
 [aFillColor release]; 
 aFillColor=[aColor retain];
}

- (void) setFillColorWarning:(NSColor*) aColor{
 [aFillColorWarning release]; 
 aFillColorWarning=[aColor retain];
}

- (void) setFillColorCritical:(NSColor*) aColor{
 [aFillColorCritical release]; 
 aFillColorCritical=[aColor retain];
}

- (void) setColorBorder:(NSColor*) aColor{
 [aBorderColor release]; 
 aBorderColor=[aColor retain];
}

-(void) setTitle:(NSString *) aString{
 [theTitle release];
 theTitle=[aString retain];
}

-(void) drawText:(NSRect)cellFrame{
 NSString *infoString = [NSString stringWithFormat:@"<span style=\"font-size: 10px; font-family: 'Lucida Grande'; color: #ffffff;\"><strong>%@</strong></span>",theTitle];
 NSAttributedString * attributedInfoString = [[NSAttributedString alloc] initWithHTML:[infoString dataUsingEncoding:NSASCIIStringEncoding] documentAttributes:NULL];
 NSPoint stringPoint;
 stringPoint.x =  cellFrame.size.width / 2 - [attributedInfoString size].width / 2;
 stringPoint.y =  cellFrame.origin.y+ (cellFrame.size.height-[attributedInfoString size].height)/2;
 [attributedInfoString drawAtPoint:stringPoint];
 [attributedInfoString release];
}

- (void)seImageForBarLeds:(NSImage *)imageNormal{
   [aImageButton release];
    aImageButton=[imageNormal retain];
}

-(void) startAnimation:(NSTimeInterval)inteval withIncrement:(double)increment{
 nInterval=inteval;
 aTimer=[NSTimer scheduledTimerWithTimeInterval:nInterval target:self selector:@selector(stepItFireMethod:) userInfo:nil repeats:YES];
 nIncrement=increment;
}

- (void)stepItFireMethod:(NSTimer*)theTimer{
  double nMinval=[self minValue];
  double nMaxval=[self maxValue];
  double nCurval=[self doubleValue];
  if(bEnableReverse==FALSE){
   nCurval+=nIncrement;
   if(nCurval > nMaxval)
    nCurval=nMinval;
  }else{
   if(nDirection==0){
    nCurval+=nIncrement;
    if(nCurval > nMaxval){
      nCurval=nMaxval;
	  nDirection=!nDirection;
	 }
   }else{
    nCurval-=nIncrement;
    if(nCurval < nMinval){
      nCurval=nMinval;
	  nDirection=!nDirection;
	 }
   }
  }
  
  [self setDoubleValue:nCurval];
  [[self controlView] setNeedsDisplay:YES];
}

-(void)stopAnimation{
 [aTimer invalidate];
 [aTimer release];
}

- (void) setEnableReverse:(BOOL) bYesNo{
 bEnableReverse=bYesNo;
}

-(void) setDoubleValue:(double)newValue{
 [super setDoubleValue:newValue];
 [[NSNotificationCenter defaultCenter] postNotificationName:@"FGHudLevelIndicatorValueChanged" object:self];
}

@end
